@@ -178,7 +178,7 @@ KateDocumentConfig::KateDocumentConfig()
       m_smartHomeSet(false),
       m_showTabsSet(false),
       m_showSpacesSet(false),
-
+      m_showSpaces(None),
       m_replaceTabsDynSet(false),
       m_removeSpacesSet(false),
       m_newLineAtEofSet(false),
@@ -223,6 +223,7 @@ KateDocumentConfig::KateDocumentConfig(const KConfigGroup &cg)
       m_smartHomeSet(false),
       m_showTabsSet(false),
       m_showSpacesSet(false),
+      m_showSpaces(None),
       m_markerSize(1),
       m_replaceTabsDynSet(false),
       m_removeSpacesSet(false),
@@ -262,6 +263,7 @@ KateDocumentConfig::KateDocumentConfig(KTextEditor::DocumentPrivate *doc)
       m_smartHomeSet(false),
       m_showTabsSet(false),
       m_showSpacesSet(false),
+      m_showSpaces(None),
       m_markerSize(1),
       m_replaceTabsDynSet(false),
       m_removeSpacesSet(false),
@@ -345,7 +347,7 @@ void KateDocumentConfig::readConfig(const KConfigGroup &config)
     setKeepExtraSpaces(config.readEntry(KEY_KEEP_EXTRA_SPACES, false));
     setIndentPastedText(config.readEntry(KEY_INDENT_PASTED_TEXT, false));
     setBackspaceIndents(config.readEntry(KEY_BACKSPACE_INDENTS, true));
-    setShowSpaces(config.readEntry(KEY_SHOW_SPACES, false));
+    setShowSpaces(KateDocumentConfig::WhitespaceRendering(config.readEntry(KEY_SHOW_SPACES, int(KateDocumentConfig::None))));
     setMarkerSize(config.readEntry(KEY_MARKER_SIZE, 1));
     setReplaceTabsDyn(config.readEntry(KEY_REPLACE_TABS_DYN, true));
     setRemoveSpaces(config.readEntry(KEY_REMOVE_SPACES, 0));
@@ -396,7 +398,7 @@ void KateDocumentConfig::writeConfig(KConfigGroup &config)
     config.writeEntry(KEY_KEEP_EXTRA_SPACES, keepExtraSpaces());
     config.writeEntry(KEY_INDENT_PASTED_TEXT, indentPastedText());
     config.writeEntry(KEY_BACKSPACE_INDENTS, backspaceIndents());
-    config.writeEntry(KEY_SHOW_SPACES, showSpaces());
+    config.writeEntry(KEY_SHOW_SPACES, int(showSpaces()));
     config.writeEntry(KEY_MARKER_SIZE, markerSize());
     config.writeEntry(KEY_REPLACE_TABS_DYN, replaceTabsDyn());
     config.writeEntry(KEY_REMOVE_SPACES, removeSpaces());
@@ -729,7 +731,7 @@ bool KateDocumentConfig::showTabs() const
     return s_global->showTabs();
 }
 
-void KateDocumentConfig::setShowSpaces(bool on)
+void KateDocumentConfig::setShowSpaces(WhitespaceRendering on)
 {
     if (m_showSpacesSet && m_showSpaces == on) {
         return;
@@ -743,7 +745,7 @@ void KateDocumentConfig::setShowSpaces(bool on)
     configEnd();
 }
 
-bool KateDocumentConfig::showSpaces() const
+KateDocumentConfig::WhitespaceRendering KateDocumentConfig::showSpaces() const
 {
     if (m_showSpacesSet || isGlobal()) {
         return m_showSpaces;
@@ -1256,6 +1258,7 @@ KateViewConfig::KateViewConfig()
     m_keywordCompletionSet(false),
     m_wordCompletionMinimalWordLengthSet(false),
     m_smartCopyCutSet(false),
+    m_mousePasteAtCursorPositionSet(false),
     m_scrollPastEndSet(false),
     m_allowMarkMenu(true),
     m_wordCompletionRemoveTailSet(false),
@@ -1306,6 +1309,7 @@ KateViewConfig::KateViewConfig(KTextEditor::ViewPrivate *view)
     m_keywordCompletionSet(false),
     m_wordCompletionMinimalWordLengthSet(false),
     m_smartCopyCutSet(false),
+    m_mousePasteAtCursorPositionSet(false),
     m_scrollPastEndSet(false),
     m_allowMarkMenu(true),
     m_wordCompletionRemoveTailSet(false),
@@ -1355,6 +1359,7 @@ const char KEY_KEYWORD_COMPLETION[] = "Keyword Completion";
 const char KEY_WORD_COMPLETION_MINIMAL_WORD_LENGTH[] = "Word Completion Minimal Word Length";
 const char KEY_WORD_COMPLETION_REMOVE_TAIL[] = "Word Completion Remove Tail";
 const char KEY_SMART_COPY_CUT[] = "Smart Copy Cut";
+const char KEY_MOUSE_PASTE_AT_CURSOR_POSITION[] = "Mouse Paste At Cursor Position";
 const char KEY_SCROLL_PAST_END[] = "Scroll Past End";
 const char KEY_FOLD_FIRST_LINE[] = "Fold First Line";
 const char KEY_SHOW_LINE_COUNT[] = "Show Line Count";
@@ -1419,6 +1424,7 @@ void KateViewConfig::readConfig(const KConfigGroup &config)
     setWordCompletionMinimalWordLength(config.readEntry(KEY_WORD_COMPLETION_MINIMAL_WORD_LENGTH, 3));
     setWordCompletionRemoveTail(config.readEntry(KEY_WORD_COMPLETION_REMOVE_TAIL, true));
     setSmartCopyCut(config.readEntry(KEY_SMART_COPY_CUT, false));
+    setMousePasteAtCursorPosition(config.readEntry(KEY_MOUSE_PASTE_AT_CURSOR_POSITION, false));
     setScrollPastEnd(config.readEntry(KEY_SCROLL_PAST_END, false));
     setFoldFirstLine(config.readEntry(KEY_FOLD_FIRST_LINE, false));
     setShowLineCount(config.readEntry(KEY_SHOW_LINE_COUNT, false));
@@ -1480,6 +1486,7 @@ void KateViewConfig::writeConfig(KConfigGroup &config)
     config.writeEntry(KEY_WORD_COMPLETION_REMOVE_TAIL, wordCompletionRemoveTail());
 
     config.writeEntry(KEY_SMART_COPY_CUT, smartCopyCut());
+    config.writeEntry(KEY_MOUSE_PASTE_AT_CURSOR_POSITION, mousePasteAtCursorPosition());
     config.writeEntry(KEY_SCROLL_PAST_END, scrollPastEnd());
     config.writeEntry(KEY_FOLD_FIRST_LINE, foldFirstLine());
 
@@ -2211,6 +2218,29 @@ void KateViewConfig::setSmartCopyCut(bool on)
     configEnd();
 }
 
+bool KateViewConfig::mousePasteAtCursorPosition() const
+{
+    if (m_mousePasteAtCursorPositionSet|| isGlobal()) {
+        return m_mousePasteAtCursorPosition;
+    }
+
+    return s_global->mousePasteAtCursorPosition();
+}
+
+void KateViewConfig::setMousePasteAtCursorPosition(bool on)
+{
+    if (m_mousePasteAtCursorPositionSet && m_mousePasteAtCursorPosition == on) {
+        return;
+    }
+
+    configStart();
+
+    m_mousePasteAtCursorPositionSet = true;
+    m_mousePasteAtCursorPosition = on;
+
+    configEnd();
+}
+
 bool KateViewConfig::scrollPastEnd() const
 {
     if (m_scrollPastEndSet || isGlobal()) {
